/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import java.util.Enumeration;
import java.util.Hashtable;
import javax.xml.transform.stream.*;
import javax.xml.transform.*;

import org.experlog.base.ESAction;
import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.gencode.DynHtml;

/**
 * XSLT transformation
 * Input parameters:
 * Xslt (MANDATORY): URL of the XSL stylesheet
 * Xml (MANDATORY): URL of the XML to transform
 * XsltCache (OPTIONAL, default false): if true, the compiled XSL stylesheet will be cached
 * XsltTimeout (OPTIONAL): max amount of time this action should take (seconds).
 * XsltDelay (OPTIONAL, default 3 x XsltTimeout if specified): delay until this action becomes
 *  active again (during the delay this action is disabled - and does nothing); The delay is
 *  activated when the timeout is reached.
 * Output parameters:
 * XsltDelayedBy: if delayed, tells until what time the delay runs (timestamp in milliseconds).
 *  0 if no delay.
 */
public class XslTransform implements ESAction {

  static Hashtable tCache_ = new Hashtable();
  static long delayedBy_ = 0; // for timeout / delay

  public boolean serverSide() { return true; }

  public boolean processRequest(Client session, ESServletRequest req,
  DynHtml expander, Object info) throws Exception {

    long crt = System.currentTimeMillis();
    if (crt < delayedBy_) {
      req.addParameter("XsltDelayedBy", Long.toString(delayedBy_));
      return true; // delayed... do nothing
    }
    req.addParameter("XsltDelayedBy", "0");
    
    // Timeout / Delay in seconds
    // If this action takes more than "Timeout", subsequent calls will be delayed.
    // Unless specified, no timeout / delay is applied.
    long timeout = req.getLongParam("XsltTimeout", -1);
    long delay = -1;
    if(timeout > 0) {
      delay = req.getLongParam("XsltDelay", 3 * timeout); // 3 x timeout by default
    }

    String xslt = req.getParameter("Xslt");
    String xml = req.getParameter("Xml");
    String cac = req.getParameter("XsltCache");
    boolean cache = (cac != null && ! "false".equalsIgnoreCase(cac));

    try {
      Transformer transformer = null;
      if(cache) transformer = (Transformer)tCache_.get(xslt);

      if(transformer == null) {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        transformer = tFactory.newTransformer(new StreamSource(xslt));
        if(cache) tCache_.put(xslt, transformer);
      }

      transformer.clearParameters();
      Enumeration pn = req.getParameterNames();
      while(pn.hasMoreElements()) {
        String nm = (String)pn.nextElement();
        if(nm.startsWith("X-") && nm.length()>2)
          transformer.setParameter(nm.substring(2), req.getParameter(nm));
      }

      transformer.transform(new StreamSource(xml), 
        new StreamResult(req.getServletResponse().getWriter()));
      
      if(timeout > 0 && System.currentTimeMillis() - crt > timeout * 1000)
        delayedBy_ = System.currentTimeMillis() + delay * 1000;

      return true;

    } catch (Exception e){
      e.printStackTrace(System.err);
      return false;
    }

  }

}
